"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RollupTemplate = exports.extractRollupConfigPathFromScript = void 0;
const path_1 = __importDefault(require("path"));
const chalk_1 = __importDefault(require("chalk"));
const find_up_1 = __importDefault(require("find-up"));
const cli_highlight_1 = __importDefault(require("cli-highlight"));
const findPackageJson_1 = require("../../../findPackageJson");
const babel = __importStar(require("@babel/core"));
function extractRollupConfigPathFromScript(script) {
    if (script.includes('rollup ')) {
        const cliArgs = script.split(' ').map((part) => part.trim());
        const configArgIndex = cliArgs.findIndex((arg) => arg === '--config' || arg === '-c');
        return configArgIndex === -1 ? null : cliArgs[configArgIndex + 1];
    }
    return null;
}
exports.extractRollupConfigPathFromScript = extractRollupConfigPathFromScript;
exports.RollupTemplate = {
    message: 'It looks like you have custom `rollup.config.js`. We can use it to bundle components for testing.',
    getExampleUrl: () => {
        return 'https://github.com/cypress-io/cypress/tree/develop/npm/react/examples/rollup';
    },
    recommendedComponentFolder: 'src',
    dependencies: ['@cypress/rollup-dev-server'],
    getPluginsCodeAst: (payload, { cypressProjectRoot }) => {
        const includeWarnComment = !payload;
        const rollupConfigPath = payload
            ? path_1.default.relative(cypressProjectRoot, payload.rollupConfigPath)
            : 'rollup.config.js';
        return {
            RequireAst: babel.template.ast([
                'const path = require("path")',
                'const { startDevServer } = require("@cypress/rollup-dev-server")',
            ].join('\n')),
            IfComponentTestingPluginsAst: babel.template.ast([
                `on("dev-server:start", async (options) => {`,
                `  return startDevServer({`,
                `    options,`,
                includeWarnComment
                    ? ' // TODO replace with valid rollup config path'
                    : '',
                `    rollupConfig: path.resolve(__dirname, '${rollupConfigPath}'),`,
                `  })`,
                `})`,
                ``,
                `return config // IMPORTANT to return the config object`,
            ].join('\n'), { preserveComments: true }),
        };
    },
    printHelper: () => {
        console.log(`Make sure that it is also required to add some additional configuration to the ${chalk_1.default.red('rollup.config.js')}. Here is whats required:`);
        const code = cli_highlight_1.default([
            `import replace from '@rollup/plugin-replace'`,
            `import commonjs from '@rollup/plugin-commonjs'`,
            `import nodeResolve from '@rollup/plugin-node-resolve'`,
            ``,
            `export default [`,
            `  {`,
            `    plugins: [`,
            `      nodeResolve(),`,
            `      // process @cypress/react-code`,
            `      commonjs(),`,
            `      // required for react sources`,
            `      replace({ 'process.env.NODE_ENV': JSON.stringify('development') }),`,
            `    ]`,
            `  }`,
            `]`,
        ].join('\n'), { language: 'js' });
        console.log(`\n${code}\n`);
    },
    test: (root) => {
        const rollupConfigPath = find_up_1.default.sync('rollup.config.js', { cwd: root });
        if (rollupConfigPath) {
            return {
                success: true,
                payload: { rollupConfigPath },
            };
        }
        const packageJsonIterator = findPackageJson_1.createFindPackageJsonIterator(root);
        return packageJsonIterator.map(({ scripts }, packageJsonPath) => {
            if (!scripts) {
                return { success: false };
            }
            for (const script of Object.values(scripts)) {
                const rollupConfigRelativePath = extractRollupConfigPathFromScript(script);
                if (rollupConfigRelativePath) {
                    const directoryRoot = path_1.default.resolve(packageJsonPath, '..');
                    const rollupConfigPath = path_1.default.resolve(directoryRoot, rollupConfigRelativePath);
                    return {
                        success: true,
                        payload: { rollupConfigPath },
                    };
                }
            }
            return { success: false };
        });
    },
};
